@@ -2,5 +2,7 @@ from .remi import REMIEncoding
 from .structured import StructuredEncoding
 from .midi_like import MIDILikeEncoding
 from .cp_word import CPWordEncoding
+from .mumidi import MuMIDIEncoding
+from .octuple import OctupleEncoding
 from .midi_tokenizer_base import quantize_note_times, detect_chords, merge_tracks, MIDITokenizer
 from .constants import MIDI_INSTRUMENTS, INSTRUMENT_CLASSES, INSTRUMENT_CLASSES_RANGES, CHORD_MAPS, DRUM_SETS
