@@ -35,22 +35,22 @@ class StructuredEncoding(MIDITokenizer):
     :param additional_tokens: specifies additional tokens (chords, empty bars, tempo...)
     :param program_tokens: will add entries for MIDI programs in the dictionary, to use
             in the case of multitrack generation for instance
+    :param params: can be a path to the parameter (json encoded) file or a dictionary
     """
     def __init__(self, pitch_range: range = PITCH_RANGE, beat_res: Dict[Tuple[int, int], int] = BEAT_RES,
                  nb_velocities: int = NB_VELOCITIES, additional_tokens: Dict[str, bool] = ADDITIONAL_TOKENS,
-                 program_tokens: bool = PROGRAM_TOKENS):
+                 program_tokens: bool = PROGRAM_TOKENS, params=None):
         # Incompatible additional tokens
         additional_tokens['Empty'] = False
         additional_tokens['Tempo'] = False
         additional_tokens['Ignore'] = False
-        super().__init__(pitch_range, beat_res, nb_velocities, additional_tokens, program_tokens)
+        super().__init__(pitch_range, beat_res, nb_velocities, additional_tokens, program_tokens, params)
 
-    def track_to_events(self, notes: List[Note], time_division: int, drum: Optional[bool] = False) -> List[Event]:
+    def track_to_events(self, track: Instrument, time_division: int) -> List[Event]:
         """ Converts a track (list of Note objects) into Event objects
 
-        :param notes: notes of the track to convert
+        :param track: track object to convert
         :param time_division: MIDI time division / resolution, in ticks/beat (of the MIDI being parsed)
-        :param drum: specify if the notes treated are from a drum track (if it is the case no chord should be detected)
         :return: list of events
                  the events should be in the order Bar -> Position -> Chord -> Pitch -> Velocity -> Duration
         """
@@ -59,11 +59,11 @@ class StructuredEncoding(MIDITokenizer):
         events = []
 
         # First time shift if needed
-        if notes[0].start != 0:
-            if notes[0].start > max(self.durations_ticks[time_division]):
-                time_shift = notes[0].start % time_division  # beat wise
+        if track.notes[0].start != 0:
+            if track.notes[0].start > max(self.durations_ticks[time_division]):
+                time_shift = track.notes[0].start % time_division  # beat wise
             else:
-                time_shift = notes[0].start
+                time_shift = track.notes[0].start
             index = np.argmin(np.abs([ticks - time_shift for ticks in self.durations_ticks[time_division]]))
             events.append(Event(
                 name='Time-Shift',
@@ -72,7 +72,7 @@ class StructuredEncoding(MIDITokenizer):
                 text=f'{time_shift} ticks'))
 
         # Creates the Pitch, Velocity, Duration and Time Shift events
-        for n, note in enumerate(notes[:-1]):
+        for n, note in enumerate(track.notes[:-1]):
             if note.pitch not in self.pitch_range:  # Notes to low or to high are discarded
                 continue
             # Pitch
@@ -97,7 +97,7 @@ class StructuredEncoding(MIDITokenizer):
                 value='.'.join(map(str, self.durations[index])),
                 text=f'{duration} ticks'))
             # Time-Shift
-            time_shift = notes[n + 1].start - note.start
+            time_shift = track.notes[n + 1].start - note.start
             index = np.argmin(np.abs([ticks - time_shift for ticks in self.durations_ticks[time_division]]))
             events.append(Event(
                 name='Time-Shift',
@@ -105,18 +105,19 @@ class StructuredEncoding(MIDITokenizer):
                 value='.'.join(map(str, self.durations[index])) if time_shift != 0 else '0.0.1',
                 text=f'{time_shift} ticks'))
         # Adds the last note
-        events.append(Event(name='Pitch', time=notes[-1].start, value=notes[-1].pitch, text=notes[-1].pitch))
-        velocity_index = (np.abs(self.velocity_bins - notes[-1].velocity)).argmin()
-        events.append(Event(name='Velocity', time=notes[-1].start, value=velocity_index,
-                            text=f'{notes[-1].velocity}/{self.velocity_bins[velocity_index]}'))
-        duration = notes[-1].end - notes[-1].start
+        events.append(Event(name='Pitch', time=track.notes[-1].start, value=track.notes[-1].pitch,
+                            text=track.notes[-1].pitch))
+        velocity_index = (np.abs(self.velocity_bins - track.notes[-1].velocity)).argmin()
+        events.append(Event(name='Velocity', time=track.notes[-1].start, value=velocity_index,
+                            text=f'{track.notes[-1].velocity}/{self.velocity_bins[velocity_index]}'))
+        duration = track.notes[-1].end - track.notes[-1].start
         index = np.argmin(np.abs([ticks - duration for ticks in self.durations_ticks[time_division]]))
-        events.append(Event(name='Duration', time=notes[-1].start, value='.'.join(map(str, self.durations[index])),
-                            text=f'{duration} ticks'))
+        events.append(Event(name='Duration', time=track.notes[-1].start,
+                            value='.'.join(map(str, self.durations[index])), text=f'{duration} ticks'))
 
         # Adds chord events if specified
-        if self.additional_tokens['Chord'] and not drum:
-            events += detect_chords(notes, time_division)
+        if self.additional_tokens['Chord'] and not track.is_drum:
+            events += detect_chords(track.notes, time_division)
 
         events.sort(key=lambda x: x.time)
 
@@ -157,7 +158,7 @@ class StructuredEncoding(MIDITokenizer):
 
         return instrument
 
-    def create_token_dicts(self, program_tokens: bool) -> Tuple[dict, dict, dict]:
+    def create_vocabulary(self, program_tokens: bool) -> Tuple[dict, dict, dict]:
         """ Create the tokens <-> event dictionaries
         These dictionaries are created arbitrary according to constants defined
         at the top of this file.
